#-------------------------------------------------------------------------------
# .PROC MyITKFilterInit
#  The "Init" procedure is called automatically by the slicer.  
#  It puts information about the module into a global array called Module, 
#  and it also initializes module-level variables.
# .ARGS
# .END
#-------------------------------------------------------------------------------
proc MyITKFilterInit {} {
    global MyITKFilter Module Volume Model

    set m MyITKFilter

    # Module Summary Info
    #------------------------------------
    # Description:
    #  Give a brief overview of what your module does, for inclusion in the 
    #  Help->Module Summaries menu item.
    set Module($m,overview) "This module is an example of how to add modules to slicer."
    #  Provide your name, affiliation and contact information so you can be 
    #  reached for any questions people may have regarding your module. 
    #  This is included in the  Help->Module Credits menu item.
    set Module($m,author) " "

    #  Set the level of development that this module falls under, from the list defined in Main.tcl,
    #  Module(categories) or pick your own
    #  This is included in the Help->Module Categories menu item
    set Module($m,category) "Example"

    # Define Tabs
    #------------------------------------
    # Description:
    #   Each module is given a button on the Slicer's main menu.
    #   When that button is pressed a row of tabs appear, and there is a panel
    #   on the user interface for each tab.  If all the tabs do not fit on one
    #   row, then the last tab is automatically created to say "More", and 
    #   clicking it reveals a second row of tabs.
    #
    #   Define your tabs here as shown below.  The options are:
    #   row1List = list of ID's for tabs. (ID's must be unique single words)
    #   row1Name = list of Names for tabs. (Names appear on the user interface
    #              and can be non-unique with multiple words.)
    #   row1,tab = ID of initial tab
    #   row2List = an optional second row of tabs if the first row is too small
    #   row2Name = like row1
    #   row2,tab = like row1 
    #
    #>>>>>>>>>>>>
    #Define Tabs for you module
    #One has to be Help
    
    set Module($m,row1List) "Help Main" 
    set Module($m,row1Name) "{Help} {Main}"
    set Module($m,row1,tab) Main

    # Define Procedures
    #------------------------------------
    # Description:
    #   The Slicer sources *.tcl files, and then it calls the Init
    #   functions of each module, followed by the VTK functions, and finally
    #   the GUI functions. A MRML function is called whenever the MRML tree
    #   changes due to the creation/deletion of nodes.
    #   
    #   While the Init procedure is required for each module, the other 
    #   procedures are optional.  If they exist, then their name (which
    #   can be anything) is registered with a line like this:
    #
    #   set Module($m,procVTK) MyITKFilterBuildVTK
    #
    #   All the options are:

    #   procGUI   = Build the graphical user interface
    #   procVTK   = Construct VTK objects
    #   procMRML  = Update after the MRML tree changes due to the creation
    #               of deletion of nodes.
    #   procEnter = Called when the user enters this module by clicking
    #               its button on the main menu
    #   procExit  = Called when the user leaves this module by clicking
    #               another modules button
    #   procCameraMotion = Called right before the camera of the active 
    #                      renderer is about to move 
    #   procStorePresets  = Called when the user holds down one of the Presets
    #               buttons.
    #               
    #   Note: if you use presets, make sure to give a preset defaults
    #   string in your init function, of the form: 
    #   set Module($m,presets) "key1='val1' key2='val2' ..."
    #   
    set Module($m,procGUI) MyITKFilterBuildGUI
    set Module($m,procVTK) MyITKFilterBuildVTK
    set Module($m,procMRML) MyITKFilterUpdateGUI
    set Module($m,procEnter) MyITKFilterEnter
    set Module($m,procExit) MyITKFilterExit

    # Define Dependencies
    #------------------------------------
    # Description:
    #   Record any other modules that this one depends on.  This is used 
    #   to check that all necessary modules are loaded when Slicer runs.
    #   
    set Module($m,depend) ""

    # Set version info
    #------------------------------------
    # Description:
    #   Record the version number for display under Help->Version Info.
    #   The strings with the $ symbol tell CVS to automatically insert the
    #   appropriate revision number and date when the module is checked in.
    #   
    lappend Module(versions) [ParseCVSInfo $m \
        {$Revision: 1.1 $} {$Date: 2004/12/10 13:27:29 $}]

    # Initialize module-level variables
    #------------------------------------
    # Description:
    #   Keep a global array with the same name as the module.
    #   This is a handy method for organizing the global variables that
    #   the procedures in this module and others need to access.
    #
   
    set MyITKFilter(inVolume) $Volume(idNone)
    set MyITKFilter(outVolume) $Volume(idNone)
    #set MyITKFilter(Model1)  $Model(idNone)
    #set MyITKFilter(FileName)  ""

    #>>>>>
    #> Check filter class params and define variable to hold these params 
    #> The itkNoiseImageFilter needs to set the Radius of the kernel.
    
    set MyITKFilter(param) "3 3 3"
}


proc MyITKFilterUpdateGUI {} {
    global MyITKFilter Volume
    
    #This commands update the list of volume nodes, so the pull down menu is uptoda
    #Check Base/tcl/tcl-shared/Developers.tcl for useful procdures like Mrml Node selection buttons.
    DevUpdateNodeSelectButton Volume MyITKFilter inVolume   inVolume   DevSelectNode
    DevUpdateNodeSelectButton Volume MyITKFilter outVolume  outVolume  DevSelectNode 0 1 1
}


# NAMING CONVENTION:
#-------------------------------------------------------------------------------
#
# Use the following starting letters for names:
# t  = toplevel
# f  = frame
# mb = menubutton
# m  = menu
# b  = button
# l  = label
# s  = slider
# i  = image
# c  = checkbox
# r  = radiobutton
# e  = entry
#
#-------------------------------------------------------------------------------


proc MyITKFilterBuildGUI {} {
    global Gui MyITKFilter Module Volume Model
    
    # A frame has already been constructed automatically for each tab.
    # A frame named "Stuff" can be referenced as follows:
    #   
    #     $Module(<Module name>,f<Tab name>)
    #
    # ie: $Module(MyITKFilter,fStuff)
    
    # This is a useful comment block that makes reading this easy for all:
    #-------------------------------------------
    # Frame Hierarchy:
    #-------------------------------------------
    # Help
    # Myframe
    #-------------------------------------------
   #-------------------------------------------
    # Help frame
    #-------------------------------------------
    
    # Write the "help" in the form of psuedo-html.  
    # Refer to the documentation for details on the syntax.
    #
    set help "
    The MyITKFilter module is an example for developers.  It shows how to add a itk Filter
    to the Slicer through ITK wrapping..
    <P>
    Description by tab:
    <BR>
    <UL>
    <LI><B>Tons o' Stuff:</B> This tab is a demo for developers.
    "
    regsub -all "\n" $help {} help

    MainHelpApplyTags MyITKFilter $help
    MainHelpBuildGUI MyITKFilter

    #-------------------------------------------
    # Myframe frame
    #-------------------------------------------
    #>>>>>>
    #>1. Get handler to the Tab frame. This is store in the Module array
    #>2. Save the handler in "fMain" variable
    
    set fMain $Module(MyITKFilter,fMain)
    
    #Definition of three frames inside the tab
    foreach frame "Top Middle Bottom" {
    frame $fMain.f$frame -bg $Gui(activeWorkspace)

    pack $fMain.f$frame -side top -padx 0 -pady $Gui(pad) -fill x
    }
    
    #-------------------------------------------
    # Myframe->Top frame
    #-------------------------------------------
    set f $fMain.fTop
    
    #       grid $f.lStuff -padx $Gui(pad) -pady $Gui(pad)
    #       grid $menubutton -sticky w
    
    # Add menus that list input and output volumes
    DevAddSelectButton  MyITKFilter $f inVolume "Input Volume" Grid
    DevAddSelectButton  MyITKFilter $f outVolume  "Output Model"  Grid
    
    # Append these menus and buttons to lists 
    # that get refreshed during UpdateMRML
    lappend Volume(mbActiveList) $f.mbinVolume
    lappend Volume(mActiveList) $f.mbinVolume.m
    lappend Volume(mbActiveList) $f.mboutVolume
    lappend Volume(mActiveList) $f.mboutVolume.m

    
    #-------------------------------------------
    # Main->Middle frame
    #-------------------------------------------
    set f $fMain.fMiddle
     
    #>>>>>>>>
    #>1. Create a label widget with parameter name ($f.l)
    eval {label $f.l -text Radius: \
          -width 12 -justify right } $Gui(WLA)
    
    #>2. Create a entry widget to get parameter ($f.e)
    eval {entry $f.e -justify right -width 10 \
          -textvariable MyITKFilter(param)  } $Gui(WEA)
    
   #>3. Pack result
   pack $f.l $f.e
   

    #-------------------------------------------
    # Main->Bottom frame
    #-------------------------------------------
    set f $fMain.fBottom

    DevAddButton $f.bApply "Apply"  "MyITKFilterApply"

    TooltipAdd $f.bApply "Apply a filter"
    pack $f.bApply 
   

}
#-------------------------------------------------------------------------------
# .PROC MyITKFilterBuildVTK
# Build any vtk objects you wish here
# .ARGS
# .END
#-------------------------------------------------------------------------------
proc MyITKFilterBuildVTK {} {

#Do you want your filter pipeine to be created when the slicer starts?
#Then, create it here.
#If not, wait until the apply method to create your pipeline

}



#-------------------------------------------------------------------------------
# .PROC MyITKFilterEnter
# Called when this module is entered by the user.  Pushes the event manager
# for this module. 
# .ARGS
# .END
#-------------------------------------------------------------------------------

proc MyITKFilterEnter {} {
    global MyITKFilter
    
    # Push event manager
    #------------------------------------
    # Description:
    #   So that this module's event bindings don't conflict with other 
    #   modules, use our bindings only when the user is in this module.
    #   The pushEventManager routine saves the previous bindings on 
    #   a stack and binds our new ones.
    #   (See slicer/program/tcl-shared/Events.tcl for more details.)
    #pushEventManager $MyITKFilter(eventManager)


}


#-------------------------------------------------------------------------------
# .PROC MyITKFilterExit
# Called when this module is exited by the user.  Pops the event manager
# for this module.  
# .ARGS
# .END
#-------------------------------------------------------------------------------
proc MyITKFilterExit {} {

    # Pop event manager
    #------------------------------------
    # Description:
    #   Use this with pushEventManager.  popEventManager removes our 
    #   bindings when the user exits the module, and replaces the 
    #   previous ones.
    #
    popEventManager
}


proc MyITKFilterApply {} {

global MyITKFilter Volume



#Get ID to volume nodes that the user has selected in the pull down menu
set v1 $MyITKFilter(inVolume)
set v2 $MyITKFilter(outVolume)

#MRML node object can be accessed by invoking methods on Volume($id,node)

#v2 is our output volume. We have to set it up
# if -5 the user has requested to create a new volume
#if not we are reusing a existing volume
if {$v2 == -5} {
  set name [Volume($v1,node) GetName]
  
  #Create a node v2 that is a copy of v1 (same dimensions and IJK to RAS transformation
  #to position the volume in RAS space.
  
  set v2 [DevCreateNewCopiedVolume $v1 ""  ${name}_filter ]
  
  #Trick to put the new node (v2) inside the same transform than v1
        set node [Volume($v2,vol) GetMrmlNode]
        Mrml(dataTree) RemoveItem $node 
        set nodeBefore [Volume($v1,vol) GetMrmlNode]
        Mrml(dataTree) InsertAfterItem $nodeBefore $node
        MainUpdateMRML
	
} else {

        set v2name  [Volume($v2,node) GetName]
        set continue [DevOKCancel "Overwrite $v2name?"]
        if {$continue == "cancel"} { return 1 }
        # They say it is OK, so overwrite!
	
	#Update v2 as v1.
        Volume($v2,node) Copy Volume($v1,node)
}
 
 
#>>>>>>>> 
#> Create filter Pipeline
#> The input vtkImageData object  can be grabbed from 
#> [Volume($v1,vol) GetOutput]
#> Don't forget that you may want to cast this volume to fit your filter.

vtkImageCast _cast
_cast SetOutputScalarTypeToFloat
_cast SetInput [Volume($v1,vol) GetOutput]
_cast Update
#Create Object

vtkITKNoiseImageFilter _filter

_filter SetInput [_cast GetOutput]
eval {_filter SetRadius} $MyITKFilter(param)

_filter AddObserver StartEvent MainStartProgress
_filter AddObserver EndEvent MainEndProgress
_filter AddObserver ProgressEvent "MainShowProgress _filter"

_filter Update

#Assign output
[Volume($v2,vol) GetOutput] DeepCopy [_filter GetOutput]

#Disconnect pipeline
_filter SetOutput ""
_filter Delete
_cast Delete


}

